#
#-###################################################################
#
# This method is for building the NYS 2D 'Mailing Address' and 'Mailing Address Overflow' fields.
# NY told us they are having problems with the way vendors are populating the 'Mailing Address Overflow' field in 
# their 2Ds. They want us to follow the MeF standard [Modernized E-File] regarding this. We were just appending the address
# fields on the document and simply truncating at the length for 'Mailing Address' and inserting what is left over, if anything,
# to the 'Mailing Address Overflow' field, which is a common technique we emply for other 2D address fields. Well, NY
# wants 'logical breaks' in the adress fields [the MeF part], so a street address with an apartment number would be split such
# that the apartment portion would go to the overflow field, or we could keep the two parts together and insert into the
# 'Mailing Address' field IF there is room. DAS has an email showing examples.
#
# Inputs:
#
# addr1: This is the main street address box.
# addr2: This is a box for the apartment number/building number/suite/etc. if there is one, else we set this argument to 'nil'.
# len1: The length of the 'Mailing Address' field from the 2D specs.
# len2: The length of the 'Mailing Address Overflow' field from the 2D specs.
#
# Returns: A two element string array, first element is for the 'Mailing Address' 2D field, the second element is for the
#	'Mailing Address Overflow' 2D field. See the 2D code for how we call this method.
#
# Logic: We are not delving into address parsing here, way too complicated. We are doing the below algorithm. The letters in
# brackets are also down in the code so you can follow the logic.
#
# [A] We first check if the combined address using 'addr1' and 'addr2' plus a separating space character if there is an 'addr2',
# will fit into the 'Mailing Address' field whose length is indicated by arg 'len1':
#
# Combined Address Fits: We set the var 'mail_addr' to this concatenation and we are done.
#
# [B] Combined Address Does Not Fit: We check if just the 'addr1' characters will fit:
#
# 	They Fit: We set the var 'mail_addr' to the address in 'addr1'. We then check if the characters in 'addr2' [if not nil] will fit
#		into the 'Mailing Address Overflow' field by checking 'len2'. [C]
#
#		They Fit: We set var 'over_addr' for the overflow portion from 'addr2'. We are done.
#
#		They Do Not Fit: We leave var 'over_addr' empty. We are done.
#
# 	[D] They Do Not Fit: Using 'len1' we look for a place to 'break' the address string in 'addr1' on a whitespace character.
#		This is as far as we are going here for a 'logical break'.
#
# 			[E] We Can Break: We now check if the remaining characters in 'addr1' plus the characters in 'addr2'
#				[if present in which case we add 1 for the space char] will fit into the 'Mailing Address Overflow'
#				field:
#
#				[F] They Fit: We set var 'over_addr' to these characters. We are done.
#
#				They DO Not Fit: We check if just the remaining characters in 'addr1' will fit.
#
#					[G] They Fit: We set var 'over_addr' to these remaining characters. We are done.
#
# 					They Do Not Fit: Var 'over_addr' is left empty. We are done.
#
#			We Can Not Break: Both vars 'addr' and 'over_addr' are left empty. We are done.
#
def get_addr_and_overflow(addr1, addr2, len1, len2)

	# We build these and return them. 'Mail Address' and 'Mail Address Overflow'.
	mail_addr = over_addr = ""

	# May as well leave if nothing to work with.
	return [mail_addr, over_addr] if ((addr1 + (addr2 ? addr2 : '')).empty?)

	# Whitespace index.
	ws_index = nil

	# [A] We add 1 for the added space character if we have a second address argument.
	if ((addr1.length + (addr2 ? addr2.length + 1 : 0)) <= len1)

		# The complete address fits in the 2D 'Mailing Address' line. We are done.
		mail_addr = "#{addr1}#{addr2 ? ' ' + addr2 : ''}".strip
	# [B]
	elsif (addr1.length <= len1)

		# We set mail_addr to addr1 since that fits.
		mail_addr = addr1

		# [C] Now we need to check if we have a second address, does it fit in the 2D 'Mailing Address Overflow' line?
		# If it does not fit we have a blank 'Mailing Address Overflow'.
		if (addr2)
			over_addr = addr2 if (addr2.length <= len2)
		end
	else
		# [D]
		# addr1 will not fit into the 2D 'Mailing Address' slot, so we attempt to break it at a word boundary. The
		# unlikely scenario where there are no whitespace characters would result in returning blank 'Mailing Address'
		# and 'Mailing Address Overflow' strings since NY does not want a blatant truncation.
		#
		if (addr1[len1-1,1] =~ /\S/)

			#
			# We have a non white space character at len1. We need to 'backup' until we see a white space character
			# and set our whitespace index var ws_index accordingly. If we can not find a whitespace char, .rindex will
			# return a nil value, which we check for further below.
			#
			ws_index = addr1[0,len1-1].rindex(/\s/)
		else
			# The address string at len1 - 1 has a white space character, so we set ws_index at len1 - 1 since indices start
			# from zero.
			ws_index = len1 - 1
		end

		#
		# [E]
		# Now we need to see if the remaining chars in addr1 plus the chars in addr2 [if we have an addr2, in which case we add 1
		# for the space char] will fit into the 2D 'Mailing Address Overflow' field whose size is indicated by len2.
		#
		if (ws_index)

			mail_addr = addr1[0, ws_index + 1]

			if ((addr1.length - (ws_index + 1) + (addr2 ? addr2.length + 1 : 0)) <= len2)

				# [F]
				over_addr = "#{addr1[ws_index + 1, addr1.length - (ws_index + 1)]}#{addr2 ? ' ' + addr2 : ''}".strip
			elsif ((addr1.length - (ws_index + 1)) <= len2)

				# [G] We only have room for the remaing chars in addr1 for the overflow.
				over_addr = addr1[ws_index + 1, addr1.length - (ws_index + 1)]
			end
		end
	end

	# Return the mailing and overflow addresses via an array.
	[mail_addr, over_addr]
end

#
#-###################################################################
#
# WorkFlow Note: "When a foreign address is entered, use the IRS abbeviation table for the country in the 2D barcode".
# This is a hash table, which basically is a collection of Key and Value pairs. The key is the country name entered by the user.
# If the key is found in the table, we retrieve the associated abbreviation. A key that is not found in the table results in a
# value of 'OC' for Other Country. We also have the United States for those smart a$$e$ out there.
#
# This table was produced by the tool country_name_abbrev_table_maker.rbw which is located in the shared folder
# off from forms programming.
#
country_hash = {"AFGHANISTAN"=>"AF","AKROTIRI"=>"AX","ALBANIA"=>"AL","ALGERIA"=>"AG","AMERICAN SAMOA"=>"AQ","ANDORRA"=>"AN","ANGOLA"=>"AO","ANGUILLA"=>"AV","ANTARCTICA"=>"AY","ANTIGUA & BARBUDA"=>"AC","ARGENTINA"=>"AR","ARMENIA"=>"AM","ARUBA"=>"AA","ASHMORE AND CARTIER ISLANDS"=>"AT","AUSTRALIA"=>"AS","AUSTRIA"=>"AU","AZERBAIJAN"=>"AJ","BAHAMAS"=>"BF","BAHRAIN"=>"BA","BAKER ISLAND"=>"FQ","BANGLADESH"=>"BG","BARBADOS"=>"BB","BELARUS"=>"BO","BELGIUM"=>"BE","BELIZE"=>"BH","BENIN"=>"BN","BERMUDA"=>"BD","BHUTAN"=>"BT","BOLIVIA"=>"BL","BOSNIAHERZEGOVINA"=>"BK","BOTSWANA"=>"BC","BOUVET ISLAND"=>"BV","BRAZIL"=>"BR","BRITISH INDIAN OCEAN TERRITORY"=>"IO","BRITISH VIRGIN ISLANDS"=>"VI","BRUNEI"=>"BX","BULGARIA"=>"BU","BURKINA FASO"=>"UV","BURMA"=>"BM","BURUNDI"=>"BY","CAMBODIA"=>"CB","CAMEROON"=>"CM","CANADA"=>"CA","CAPE VERDE"=>"CV","CAYMAN ISLANDS"=>"CJ","CENTRAL AFRICAN REPUBLIC"=>"CT","CHAD"=>"CD","CHILE"=>"CI","CHINA"=>"CH","CHRISTMAS ISLAND"=>"KT","CLIPPERTON ISLAND"=>"IP","COCOS (KEELING) ISLANDS"=>"CK","COCOS ISLANDS"=>"CK","KEELING ISLANDS"=>"CK","COLOMBIA"=>"CO","COMOROS"=>"CN","CONGO (BRAZZAVILLE)"=>"CF","CONGO (KINSHASA)"=>"CG","COOK ISLANDS"=>"CW","CORAL SEA ISLANDS"=>"CR","COSTA RICA"=>"CS","COTE D'IVOIRE (IVORY COAST)"=>"IV","COTE D'IVOIRE"=>"IV","IVORY COAST"=>"IV","CROATIA"=>"HR","CUBA"=>"CU","CURACAO"=>"UC","CYPRUS"=>"CY","CZECH REPUBLIC"=>"EZ","DENMARK"=>"DA","DHEKELIA"=>"DX","DJIBOUTI"=>"DJ","DOMINICA"=>"DO","DOMINICAN REPUBLIC"=>"DR","EAST TIMOR"=>"TT","ECUADOR"=>"EC","EGYPT"=>"EG","EL SALVADOR"=>"ES","EQUATORIAL GUINEA"=>"EK","ERITREA"=>"ER","ESTONIA"=>"EN","ETHIOPIA"=>"ET","FALKLAND ISLANDS (ISLAS MALVINAS)"=>"FK","FALKLAND ISLANDS"=>"FK","ISLAS MALVINAS"=>"FK","FAROE ISLANDS"=>"FO","FEDERATED STATES OF MICRONESIA"=>"FM","FIJI"=>"FJ","FINLAND"=>"FI","FRANCE"=>"FR","FRENCH POLYNESIA"=>"FP","FRENCH SOUTHERN AND ANTARCTIC LANDS"=>"FS","GABON"=>"GB","THE GAMBIA"=>"GA","GEORGIA"=>"GG","GERMANY"=>"GM","GHANA"=>"GH","GIBRALTAR"=>"GI","GREECE"=>"GR","GREENLAND"=>"GL","GRENADA"=>"GJ","GUAM"=>"GQ","GUATEMALA"=>"GT","GUERNSEY"=>"GK","GUINEA"=>"GV","GUINEABISSAU"=>"PU","GUYANA"=>"GY","HAITI"=>"HA","HEARD ISLAND AND MCDONALD ISLANDS"=>"HM","HOLY SEE"=>"VT","HONDURAS"=>"HO","HONG KONG"=>"HK","HOWLAND ISLAND"=>"HQ","HUNGARY"=>"HU","ICELAND"=>"IC","INDIA"=>"IN","INDONESIA"=>"ID","IRAN"=>"IR","IRAQ"=>"IZ","IRELAND"=>"EI","ISRAEL"=>"IS","ITALY"=>"IT","JAMAICA"=>"JM","JAN MAYEN"=>"JN","JAPAN"=>"JA","JARVIS ISLAND"=>"DQ","JERSEY"=>"JE","JOHNSTON ATOLL"=>"JQ","JORDAN"=>"JO","KAZAKHSTAN"=>"KZ","KENYA"=>"KE","KINGMAN REEF"=>"KQ","KIRIBATI"=>"KR","KOREA, DEMOCRATIC PEOPLE'S REPUBLIC OF (NORTH)"=>"KN","DEMOCRATIC PEOPLE'S REPUBLIC OF NORTH KOREA"=>"KN","NORTH KOREA"=>"KN","KOREA, REPUBLIC OF (SOUTH)"=>"KS","REPUBLIC OF SOUTH KOREA"=>"KS","SOUTH KOREA"=>"KS","KOSOVO"=>"KV","KUWAIT"=>"KU","KYRGYZSTAN"=>"KG","LAOS"=>"LA","LATVIA"=>"LG","LEBANON"=>"LE","LESOTHO"=>"LT","LIBERIA"=>"LI","LIBYA"=>"LY","LIECHTENSTEIN"=>"LS","LITHUANIA"=>"LH","LUXEMBOURG"=>"LU","MACAU"=>"MC","MACEDONIA"=>"MK","MADAGASCAR"=>"MA","MALAWI"=>"MI","MALAYSIA"=>"MY","MALDIVES"=>"MV","MALI"=>"ML","MALTA"=>"MT","MAN, ISLE OF"=>"IM","ISLE OF MAN"=>"IM","MARSHALL ISLANDS"=>"RM","MAURITANIA"=>"MR","MAURITIUS"=>"MP","MEXICO"=>"MX","MIDWAY ISLANDS"=>"MQ","MOLDOVA"=>"MD","MONACO"=>"MN","MONGOLIA"=>"MG","MONTENEGRO"=>"MJ","MONTSERRAT"=>"MH","MOROCCO"=>"MO","MOZAMBIQUE"=>"MZ","NAMIBIA"=>"WA","NAURU"=>"NR","NAVASSA ISLAND"=>"BQ","NEPAL"=>"NP","NETHERLANDS"=>"NL","NEW CALEDONIA"=>"NC","NEW ZEALAND"=>"NZ","NICARAGUA"=>"NU","NIGER"=>"NG","NIGERIA"=>"NI","NIUE"=>"NE","NORFOLK ISLAND"=>"NF","NORTHERN MARIANA ISLANDS"=>"CQ","NORWAY"=>"NO","OMAN"=>"MU","OTHER COUNTRY"=>"OC","OTHER"=>"OC","PAKISTAN"=>"PK","PALAU"=>"PS","PALMYRA ATOLL"=>"LQ","PANAMA"=>"PM","PAPUANEW GUINEA"=>"PP","PARACEL ISLANDS"=>"PF","PARAGUAY"=>"PA","PERU"=>"PE","PHILIPPINES"=>"RP","PITCAIRN ISLANDS"=>"PC","POLAND"=>"PL","PORTUGAL"=>"PO","PUERTO RICO"=>"RQ","QATAR"=>"QA","ROMANIA"=>"RO","RUSSIA"=>"RS","RWANDA"=>"RW","SAINT BARTHELEMY"=>"TB","SAINT MARTIN"=>"RN","SAMOA"=>"WS","SAN MARINO"=>"SM","SAO TOME AND PRINCIPE"=>"TP","SAUDI ARABIA"=>"SA","SENEGAL"=>"SG","SERBIA"=>"RI","SEYCHELLES"=>"SE","SIERRA LEONE"=>"SL","SINGAPORE"=>"SN","SINT MAARTEN"=>"NN","SLOVAKIA"=>"LO","SLOVENIA"=>"SI","SOLOMON ISLANDS"=>"BP","SOMALIA"=>"SO","SOUTH AFRICA"=>"SF","SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS"=>"SX","SOUTH SUDAN"=>"OD","SPAIN"=>"SP","SPRATLY ISLANDS"=>"PG","SRI LANKA"=>"CE","ST. HELENA"=>"SH","ST. KITTS AND NEVIS"=>"SC","ST. LUCIA ISLAND"=>"ST","ST. PIERRE AND MIQUELON"=>"SB","ST. VINCENT AND THE GRENADINES"=>"VC","SUDAN"=>"SU","SURINAME"=>"NS","SVALBARD"=>"SV","SWAZILAND"=>"WZ","SWEDEN"=>"SW","SWITZERLAND"=>"SZ","SYRIA"=>"SY","TAIWAN"=>"TW","TAJIKISTAN"=>"TI","TANZANIA"=>"TZ","THAILAND"=>"TH","TOGO"=>"TO","TOKELAU"=>"TL","TONGA"=>"TN","TRINIDAD AND TOBAGO"=>"TD","TUNISIA"=>"TS","TURKEY"=>"TU","TURKMENISTAN"=>"TX","TURKS AND CAICOS ISLANDS"=>"TK","TUVALU"=>"TV","UGANDA"=>"UG","UKRAINE"=>"UP","UNITED ARAB EMIRATES"=>"AE","UNITED KINGDOM (ENGLAND, NORTHERN IRELAND, SCOTLAND, AND WALES)"=>"UK","UNITED KINGDOM"=>"UK","ENGLAND"=>"UK","NORTHERN IRELAND"=>"UK","SCOTLAND"=>"UK","WALES"=>"UK","URUGUAY"=>"UY","UZBEKISTAN"=>"UZ","VANUATU"=>"NH","VENEZUELA"=>"VE","VIETNAM"=>"VM","VIRGIN ISLANDS"=>"VQ","WAKE ISLAND"=>"WQ","WALLIS AND FUTUNA"=>"WF","WESTERN SAHARA"=>"WI","YEMEN (ADEN)"=>"YM","ZAMBIA"=>"ZA","ZIMBABWE"=>"ZI","USA"=>"US","UNITED STATES OF AMERICA"=>"US"}

#
#-###################################################################
#

#
# NY wants us to follow MeF standards for handling Address field overflow for the Mailing Address, Permanent Address, and
# Preparer's Address fields. We are using the method get_addr_and_overflow( ) for this. We are using vars for this, and will
# set them here using parallel assignment. The FIRST method argument is for the street address box. The SECOND method
# argument is for the Apt/Unit/Suite..etc. address box. If an address has no separate box for Apt/Unit/Suite..etc. set the
# second method argument to the value 'nil' [minus the single quotes]. The THIRD argument is for the 2D length of the first
# address field. The FOURTH [last] argument is for the 2D length of the second/overflow address field. 
#
mail_addr, mail_addr_over = get_addr_and_overflow(box2_11.delete(bad_addr_chars).strip, box2_12.delete(bad_addr_chars).strip, 30, 30)

perm_addr, perm_addr_over = get_addr_and_overflow(box2_19.delete(bad_addr_chars).strip, box2_92.delete(bad_addr_chars).strip, 30, 30)

# 2D specs for preparer address filters are different than those for other addresses.
prep_addr, prep_addr_over = get_addr_and_overflow(box5_58.delete("^A-Z0-9 ").strip, box5_38.delete("^A-Z0-9 ").strip, 30, 30)

# Down in the 2D we just do this....just an example of the mailing address
mail_addr << "\r" <<
mail_addr_over << "\r" <<
